/*******************************************************************************
* File Name: SleepTimer.c
* Version 1.10
*
*  Description:
*   This file provides the source code to API for the Sleep Timer.
*
*  Note:
*
********************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/ 


#include "SleepTimer.h"

/* Take parameters from:
 *  xxxxxx00 - No Interval, nor EnableInt was changed by API
 *  xxxxxx01 - No Interval, but EnableInt was changed by API
 *  xxxxxx10 - Interval, but not EnableInt was changed by API
 *  xxxxxx11 - Interval and EnableInt was changed by API
*/
uint8 SleepTimer_flags = 0;


/*******************************************************************************
* Function Name: SleepTimer_Start
********************************************************************************
*
* Summary:
*  Starts the Sleep Timer operation: enables 1 kHz clock, sets default
*  parameters if they were not set previously by corresponding API functions.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Side Effects:
*  Enables 1 kHz ILO clocks and leaves it enabled after Sleep Time component
*  is stopped.
*
*******************************************************************************/
void SleepTimer_Start(void)
{
    /* Check if 1kHz ILO is disabled */
    if (0 == (SleepTimer_ILO_CFG & SleepTimer_ILO_1KHZ_EN))
    {
        SleepTimer_ILO_CFG |=SleepTimer_ILO_1KHZ_EN;
    }
    /* If 1kHz ILO is enabled - do nothing */


    /* Sets default (passed by parameter from customizer) interval if it was
     *   not manually set by API function                                       */
    if (0 == (SleepTimer_flags & SleepTimer_INTERVAL_CHNG_BIT))
    {
        SleepTimer_SetInterval(SleepTimer_INTERVAL);
    }
    /* Interval is already set */


    /* Sets default (customizer's) interrupt enable if it was not set by API  */
    if (0 == (SleepTimer_flags & SleepTimer_INTERRUPT_CHNG_BIT))
    {
        /* Check if user sets to issue an interrupt in customizer */
        if ( 1 == SleepTimer_ENABLE_INTERRUPT )
        {
            /* Enable interrupt */
            SleepTimer_EnableInt();
        } else
        {
            /* Disable interrupt */
            SleepTimer_DisableInt();
        }
    }
    /* Interrupt enable/disable is already applied */

    /* Enable wake up on CTW counter */
    SleepTimer_TW_CFG |= SleepTimer_CTW_EN;

}


/*******************************************************************************
* Function Name: SleepTimer_Stop
********************************************************************************
*
* Summary:
*  Stops the Sleep Timer operation: disables wake up on CTW counter terminal
*  count reached and disables interrupt issuing on foregoing condition.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Side Effects:
* All changes made by API to configuration of SleepTimer before this function
* was called will not be used, values from customizer will be used instead, when
* SleepTimer_Start() will be executed next time.
*
*******************************************************************************/
void SleepTimer_Stop(void)
{
    /* Disable wake up on CTW counter */
    SleepTimer_TW_CFG &= ~SleepTimer_CTW_EN;
    /* Just for sure of next time start */
    SleepTimer_TW_CFG &= ~SleepTimer_CTW_IE;
    /* Clear flag of changes was made by API */
    SleepTimer_flags = 0;
}


/*******************************************************************************
* Function Name: SleepTimer_EnableInt
********************************************************************************
*
* Summary:
*  Enables Sleep Timer component issuing interrupt on wake up.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void SleepTimer_EnableInt(void)
{

     /* Update flag that interrupt was set by API */
    SleepTimer_flags |= SleepTimer_INTERRUPT_CHNG_BIT;
     /* Enable interrupt on wake up */
    SleepTimer_TW_CFG |= SleepTimer_CTW_IE;
}


/*******************************************************************************
* Function Name: SleepTimer_DisableInt
********************************************************************************
*
* Summary:
*  Disables Sleep Timer component issuing interrupt on wake up.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void SleepTimer_DisableInt(void)
{
    /* Update flag that interrupt was set by API */
    SleepTimer_flags |= SleepTimer_INTERRUPT_CHNG_BIT;
    /* Disable interrupt on wake up */
    SleepTimer_TW_CFG &= ~SleepTimer_CTW_IE;
}


/*******************************************************************************
* Function Name: SleepTimer_SetInterval
********************************************************************************
*
* Summary:
*  Sets interval for Sleep Timer to wake up.
*
* Parameters:
*  interval:  This parameter is used to define interval value in milliseconds.
*  Acceptable is only values from a fixed range:
*   Name                            Value        Period
*   SleepTimer__CTW_2_MS      4'b0001        2 ms
*   SleepTimer__CTW_4_MS      4'b0010        4 ms
*   SleepTimer__CTW_8_MS      4'b0011        8 ms
*   SleepTimer__CTW_16_MS     4'b0100        16 ms
*   SleepTimer__CTW_32_MS     4'b0101        32 ms
*   SleepTimer__CTW_64_MS     4'b0110        64 ms
*   SleepTimer__CTW_128_MS    4'b0111        128 ms
*   SleepTimer__CTW_256_MS    4'b1000        256 ms
*   SleepTimer__CTW_512_MS    4'b1001        512 ms
*   SleepTimer__CTW_1024_MS   4'b1010        1024 ms
*   SleepTimer__CTW_2048_MS   4'b1011        2048 ms
*   SleepTimer__CTW_4096_MS   4'b1100        4096 ms
*
* Return:
*  void
*
* Side Effects:
*  The first interval can range from 1 to (period + 1)ms. Additional intervals
*  occur at the nominal period. Interval value can be only changed when
*  Sleep Timer is stopped.
*
*******************************************************************************/
void SleepTimer_SetInterval(uint8 interval)
{
    /* Check if CTW is disabled */
    if (0 == (SleepTimer_TW_CFG & SleepTimer_CTW_EN))
    {
        /* Clear interval setting */
        SleepTimer_CTW_INTERVAL &= ~ SleepTimer_INTERVAL_MASK;
        /* Set wake up interval */
        SleepTimer_CTW_INTERVAL |= \
            (interval & SleepTimer_INTERVAL_MASK);
        /* Update flag that interval was set by API */
        SleepTimer_flags |= SleepTimer_INTERVAL_CHNG_BIT;
    }
    /* Do nothing if CTW is enabled. */

}


/*******************************************************************************
* Function Name: SleepTimer_GetStatus
********************************************************************************
*
* Summary:
*  Clears all interrupt status bits (react_int, limact_int, onepps_int, ctw_int,
*  and  ftw_int)  in Power Manager Interrupt Status Register. This function 
*  should always be called (when Sleep Timer?s interrupt is disabled or enabled)
*  after wake up to clear ctw_int status bit.
*
* Parameters:
*  void
*
* Return:
* uint8 with bits set if corresponding event has occurred
* Name                               Value   Description
*SleepTimer_PM_INT_SR_REACT    0x80u   A transition from limited active to 
*                                            active mode event occured 
*SleepTimer_PM_INT_SR_LIMACT   0x08u   Limited active ready event occured
*SleepTimer_PM_INT_SR_ONEPPSP  0x04u   A onepps event has occured
*SleepTimer_PM_INT_SR_CTW      0x02u   A central timewheel event occured
*SleepTimer_PM_INT_SR_FTW      0x01u   A fast timewheel event occured
*
* Side Effects:
*  If an interrupt gets generated at the same time as a clear, the bit will 
*  remain set (which causes another interrupt).
*
*******************************************************************************/
uint8 SleepTimer_GetStatus(void)
{
    uint8 status;

    /* Read to clear  */
    status = SleepTimer_INT_SR;

    return status;
}

/* [] END OF FILE */
