/*******************************************************************************
* File Name: Timer_Delay.c  
* Version 2.0
*
* Description:
*  The Timer User Module consists of a 8, 16, 24 or 32-bit timer with
*  a selectable period between 2 and 2^Width - 1.  The timer may free run
*  or be used as a capture timer as well.  The capture can be initiated
*  by a positive or negative edge signal as well as via software.
*  A trigger input can be programmed to enable the timer on rising edge
*  falling edge, either edge or continous run.
*  Interrupts may be generated due to a terminal count condition
*  or a capture event.
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Timer_Delay.h"

uint8 Timer_Delay_initVar = 0u;


/*******************************************************************************
* Function Name: Timer_Delay_Init
********************************************************************************
* 
* Summary:
*  Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void Timer_Delay_Init(void) 
{
    #if(!Timer_Delay_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 Timer_Delay_interruptState;
    #endif

    #if (Timer_Delay_UsingFixedFunction)
        /* Clear all bits but the enable bit (if it's already set) for Timer operation */
        Timer_Delay_CONTROL &= Timer_Delay_CTRL_ENABLE;
        
        /* Clear the mode bits for continuous run mode */
        #if (Timer_Delay_PSOC3_ES2 || Timer_Delay_PSOC5_ES1)
            Timer_Delay_CONTROL2 &= ~Timer_Delay_CTRL_MODE_MASK;
        #endif
        #if (Timer_Delay_PSOC3_ES3 || Timer_Delay_PSOC5_ES2)
            Timer_Delay_CONTROL3 &= ~Timer_Delay_CTRL_MODE_MASK;                
        #endif

        /* Check if One Shot mode is enabled i.e. RunMode !=0*/
        #if (Timer_Delay_RunModeUsed != 0x0u)
            /* Set 3rd bit of Control register to enable one shot mode */
            Timer_Delay_CONTROL |= 0x04u;
        #endif
        
        #if (Timer_Delay_RunModeUsed == 2)
            #if (Timer_Delay_PSOC3_ES2 || Timer_Delay_PSOC5_ES1)
                /* Set last 2 bits of control2 register if one shot(halt on 
                interrupt) is enabled*/
                Timer_Delay_CONTROL2 |= 0x03u;
            #endif
            #if (Timer_Delay_PSOC3_ES3 || Timer_Delay_PSOC5_ES2)
                /* Set last 2 bits of control3 register if one shot(halt on 
                interrupt) is enabled*/
                Timer_Delay_CONTROL3 |= 0x03u;                
            #endif
        #endif
        
        #if (Timer_Delay_UsingHWEnable != 0)
            #if (Timer_Delay_PSOC3_ES2 || Timer_Delay_PSOC5_ES1)
                /* Set the default Run Mode of the Timer to Continuous */
                Timer_Delay_CONTROL2 |= Timer_Delay_CTRL_MODE_PULSEWIDTH;
            #endif
            #if (Timer_Delay_PSOC3_ES3 || Timer_Delay_PSOC5_ES2)
                /* Clear and Set ROD and COD bits of CFG2 register */
                Timer_Delay_CONTROL3 &= ~Timer_Delay_CTRL_RCOD_MASK;
                Timer_Delay_CONTROL3 |= Timer_Delay_CTRL_RCOD;  
                
                /* Clear and Enable the HW enable bit in CFG2 register */
                Timer_Delay_CONTROL3 &= ~Timer_Delay_CTRL_ENBL_MASK;
                Timer_Delay_CONTROL3 |= Timer_Delay_CTRL_ENBL;     
                
                /* Set the default Run Mode of the Timer to Continuous */
                Timer_Delay_CONTROL3 |= Timer_Delay_CTRL_MODE_CONTINUOUS;
            #endif
        #endif
        
        /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
        Timer_Delay_RT1 &= ~Timer_Delay_RT1_MASK;
        Timer_Delay_RT1 |= Timer_Delay_SYNC;     
                
        /*Enable DSI Sync all all inputs of the Timer*/
        Timer_Delay_RT1 &= ~(Timer_Delay_SYNCDSI_MASK);
        Timer_Delay_RT1 |= Timer_Delay_SYNCDSI_EN;
                
        /* Set the IRQ to use the status register interrupts */
        Timer_Delay_CONTROL2 |= Timer_Delay_CTRL2_IRQ_SEL;
    #endif 
        
    /* Set Initial values from Configuration */
    Timer_Delay_WritePeriod(Timer_Delay_INIT_PERIOD);
    Timer_Delay_WriteCounter(Timer_Delay_INIT_PERIOD);
    
    #if (Timer_Delay_UsingHWCaptureCounter)/* Capture counter is enabled */
        Timer_Delay_CAPTURE_COUNT_CTRL |= Timer_Delay_CNTR_ENABLE;
        Timer_Delay_SetCaptureCount(Timer_Delay_INIT_CAPTURE_COUNT);
    #endif
        
    #if (!Timer_Delay_UsingFixedFunction)
        #if (Timer_Delay_SoftwareCaptureMode)
            Timer_Delay_SetCaptureMode(Timer_Delay_INIT_CAPTURE_MODE);
        #endif
        
        #if (Timer_Delay_SoftwareTriggerMode)
            if (!(Timer_Delay_CONTROL & Timer_Delay__B_TIMER__TM_SOFTWARE))
            {
                Timer_Delay_SetTriggerMode(Timer_Delay_INIT_TRIGGER_MODE);
            }
        #endif
        
        /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
        /* Enter Critical Region*/
        Timer_Delay_interruptState = CyEnterCriticalSection();
        
        /* Use the interrupt output of the status register for IRQ output */
        Timer_Delay_STATUS_AUX_CTRL |= Timer_Delay_STATUS_ACTL_INT_EN_MASK;
        
        /* Exit Critical Region*/
        CyExitCriticalSection(Timer_Delay_interruptState);

        #if (Timer_Delay_EnableTriggerMode)
            Timer_Delay_EnableTrigger();
        #endif

        #if (Timer_Delay_InterruptOnCaptureCount)
             #if (!Timer_Delay_ControlRegRemoved)
                Timer_Delay_SetInterruptCount(Timer_Delay_INIT_INT_CAPTURE_COUNT);
            #endif
        #endif
        
        Timer_Delay_ClearFIFO();  
    #endif

    Timer_Delay_SetInterruptMode(Timer_Delay_INIT_INTERRUPT_MODE);
}


/*******************************************************************************
* Function Name: Timer_Delay_Enable
********************************************************************************
* 
* Summary:
*  Enable the Timer
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void Timer_Delay_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (Timer_Delay_UsingFixedFunction)
        Timer_Delay_GLOBAL_ENABLE |= Timer_Delay_BLOCK_EN_MASK;
        Timer_Delay_GLOBAL_STBY_ENABLE |= Timer_Delay_BLOCK_STBY_EN_MASK;
    #endif   
    
    /* Remove assignment if control register is removed */
    #if (!Timer_Delay_ControlRegRemoved || Timer_Delay_UsingFixedFunction)     
        Timer_Delay_CONTROL |= Timer_Delay_CTRL_ENABLE;
    #endif
}


/*******************************************************************************
* Function Name: Timer_Delay_Start
********************************************************************************
* 
* Summary:
*  The start function initializes the timer with the default values, the 
*  enables the timerto begin counting.  It does not enable interrupts,
*  the EnableInt command should be called if interrupt generation is required.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  Timer_Delay_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
* Reentrant
*  No
*
*******************************************************************************/
void Timer_Delay_Start(void) 
{
    if(Timer_Delay_initVar == 0u)
    {
        Timer_Delay_Init();
        
        Timer_Delay_initVar = 1u;   /* Clear this bit for Initialization */
    }
    
    /* Enable the Timer */		
    Timer_Delay_Enable();    
}


/*******************************************************************************
* Function Name: Timer_Delay_Stop
********************************************************************************
* 
* Summary:
*  The stop function halts the timer, but does not change any modes or disable
*  interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the timer.
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_Stop(void) 
{
    /* Disable Timer */
    #if(!Timer_Delay_ControlRegRemoved || Timer_Delay_UsingFixedFunction)    /* Remove assignment if control register is removed */  
        Timer_Delay_CONTROL &= ~Timer_Delay_CTRL_ENABLE;
    #endif  
    
    /* Globally disable the Fixed Function Block chosen */
    #if (Timer_Delay_UsingFixedFunction)
        Timer_Delay_GLOBAL_ENABLE &= ~Timer_Delay_BLOCK_EN_MASK;
        Timer_Delay_GLOBAL_STBY_ENABLE &= ~Timer_Delay_BLOCK_STBY_EN_MASK;
    #endif
}


/*******************************************************************************
* Function Name: Timer_Delay_SetInterruptMode
********************************************************************************
* 
* Summary:
*  This function selects which of the interrupt inputs may cause an interrupt.  
*  The twosources are caputure and terminal.  One, both or neither may 
*  be selected.
*
* Parameters:  
*  interruptMode:   This parameter is used to enable interrups on either/or 
*                   terminal count or capture.  
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_SetInterruptMode(uint8 interruptMode) 
{
    Timer_Delay_STATUS_MASK = interruptMode;
}

/*******************************************************************************
* Function Name: Timer_Delay_SoftwareCapture
********************************************************************************
* 
* Summary:
*  This function forces a capture independent of the capture signal.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Side Effects:
*  An existing hardware capture could be overwritten.
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_SoftwareCapture(void) 
{
    /* Generate a software capture by reading the counter register */
    CY_GET_REG8(Timer_Delay_COUNTER_LSB_PTR);
    /* Capture Data is now in the FIFO */
}


/*******************************************************************************
* Function Name: Timer_Delay_ReadStatusRegister
********************************************************************************
* 
* Summary:
*  Reads the status register and returns it's state. This function should use
*  defined types for the bit-field information as the bits in this register may
*  be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  The contents of the status register
*
* Side Effects:
*  Status register bits may be clear on read. 
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8   Timer_Delay_ReadStatusRegister(void) 
{
    return Timer_Delay_STATUS;
}


#if (!Timer_Delay_ControlRegRemoved)   /* Remove API if control register is removed */
/*******************************************************************************
* Function Name: Timer_Delay_ReadControlRegister
********************************************************************************
* 
* Summary:
*  Reads the control register and returns it's value. 
*
* Parameters:  
*  void
*
* Return: 
*  The contents of the control register
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8 Timer_Delay_ReadControlRegister(void) 
{
    return Timer_Delay_CONTROL;
}


/*******************************************************************************
* Function Name: Timer_Delay_WriteControlRegister
********************************************************************************
* 
* Summary:
*  Sets the bit-field of the control register.  
*
* Parameters:  
*  control: The contents of the control register
*
* Return: 
*  
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_WriteControlRegister(uint8 control) 
{
    Timer_Delay_CONTROL = control;
}
#endif /* Remove API if control register is removed */


/*******************************************************************************
* Function Name: Timer_Delay_ReadPeriod
********************************************************************************
* 
* Summary:
*  This function returns the current value of the Period.
*
* Parameters:  
*  void
*
* Return: 
*  The present value of the counter.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint16 Timer_Delay_ReadPeriod(void) 
{
   return ( CY_GET_REG16(Timer_Delay_PERIOD_LSB_PTR) );  
}


/*******************************************************************************
* Function Name: Timer_Delay_WritePeriod
********************************************************************************
* 
* Summary:
*  This function is used to change the period of the counter.  The new period 
*  will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: This value may be between 1 and (2^Resolution)-1.  A value of 0 will result in
*          the counter remaining at zero.
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_WritePeriod(uint16 period) 
{
    #if(Timer_Delay_UsingFixedFunction)
        uint16 period_temp = (uint16)period;
        CY_SET_REG16(Timer_Delay_PERIOD_LSB_PTR, period_temp);
    #else
        CY_SET_REG16(Timer_Delay_PERIOD_LSB_PTR, period);
    #endif
}


/*******************************************************************************
* Function Name: Timer_Delay_ReadCapture
********************************************************************************
* 
* Summary:
*  This function returns the last value captured.
*
* Parameters:  
*  void 
*
* Return: 
*  Present Capture value.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint16 Timer_Delay_ReadCapture(void) 
{
   return ( CY_GET_REG16(Timer_Delay_CAPTURE_LSB_PTR) );  
}


/*******************************************************************************
* Function Name: Timer_Delay_WriteCounter
********************************************************************************
* 
* Summary:
*  This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_WriteCounter(uint16 counter) 
{
    #if(Timer_Delay_UsingFixedFunction)
        counter = counter;
        /* This functionality is removed until a FixedFunction HW update to 
         * allow this register to be written 
         */
        /* uint16 counter_temp = (uint16)counter;
         * CY_SET_REG16(Timer_Delay_COUNTER_LSB_PTR, counter_temp);
         */
    #else
        CY_SET_REG16(Timer_Delay_COUNTER_LSB_PTR, counter);
    #endif
}


/*******************************************************************************
* Function Name: Timer_Delay_ReadCounter
********************************************************************************
* 
* Summary:
*  This function returns the current counter value.
*
* Parameters:  
*  void
*
* Return: 
*  Present compare value. 
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint16 Timer_Delay_ReadCounter(void) 
{
    
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    CY_GET_REG8(Timer_Delay_COUNTER_LSB_PTR);
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    return (CY_GET_REG16(Timer_Delay_CAPTURE_LSB_PTR));
}


#if(!Timer_Delay_UsingFixedFunction) /* UDB Specific Functions */

/*******************************************************************************
 * The functions below this point are only available using the UDB 
 * implementation.  If a feature is selected, then the API is enabled.
 ******************************************************************************/


#if (Timer_Delay_SoftwareCaptureMode)
/*******************************************************************************
* Function Name: Timer_Delay_SetCaptureMode
********************************************************************************
* 
* Summary:
*  This function sets the capture mode to either rising or falling edge.
*
* Parameters:  
*  captureMode:  This parameter sets the capture mode of the UDB capture feature.
*  The parameter values are defined using the #define Timer_Delay__B_TIMER__CM_NONE 0
#define Timer_Delay__B_TIMER__CM_RISINGEDGE 1
#define Timer_Delay__B_TIMER__CM_FALLINGEDGE 2
#define Timer_Delay__B_TIMER__CM_EITHEREDGE 3
#define Timer_Delay__B_TIMER__CM_SOFTWARE 4
. 
*  The following are the possible values of the parameter
*  Timer_Delay__B_TIMER__CM_NONE        - Set Capture mode to None
*  Timer_Delay__B_TIMER__CM_RISINGEDGE  - Capture on rising edge of Capture input
*  Timer_Delay__B_TIMER__CM_FALLINGEDGE - Capture on falling edge of Capture input
*  Timer_Delay__B_TIMER__CM_EITHEREDGE  - Capture on falling edge of Capture input
* 
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_SetCaptureMode(uint8 captureMode) 
{
    /* This must only set to two bits of the control register associated */
    captureMode = (captureMode << Timer_Delay_CTRL_CAP_MODE_SHIFT);
    captureMode &= (Timer_Delay_CTRL_CAP_MODE_MASK);
    
	/* Clear the Current Setting */
    Timer_Delay_CONTROL &= ~Timer_Delay_CTRL_CAP_MODE_MASK;
    
    /* Write The New Setting */   
    Timer_Delay_CONTROL |= captureMode;
}
#endif

#if (Timer_Delay_SoftwareTriggerMode)
/*******************************************************************************
* Function Name: Timer_Delay_SetTriggerMode
********************************************************************************
* 
* Summary:
*  This function sets the trigger input mode
*
* Parameters:  
*  triggerMode: Pass one of the pre-defined Trigger Modes (except Software)
    #define Timer_Delay__B_TIMER__TM_NONE 0x00
    #define Timer_Delay__B_TIMER__TM_RISINGEDGE 0x04
    #define Timer_Delay__B_TIMER__TM_FALLINGEDGE 0x08
    #define Timer_Delay__B_TIMER__TM_EITHEREDGE 0x0C
    #define Timer_Delay__B_TIMER__TM_SOFTWARE 0x10
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_SetTriggerMode(uint8 triggerMode) 
{
    /* This must only set to two bits of the control register associated */
    triggerMode &= Timer_Delay_CTRL_TRIG_MODE_MASK;
    
    /* Clear the Current Setting */
    Timer_Delay_CONTROL &= ~Timer_Delay_CTRL_TRIG_MODE_MASK;
    
    /* Write The New Setting */   
    Timer_Delay_CONTROL |= (triggerMode | Timer_Delay__B_TIMER__TM_SOFTWARE);     

}
#endif

#if (Timer_Delay_EnableTriggerMode)
/*******************************************************************************
* Function Name: Timer_Delay_EnableTrigger
********************************************************************************
* 
* Summary:
*  Sets the control bit enabling Hardware Trigger mode
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_EnableTrigger(void) 
{
    #if (!Timer_Delay_ControlRegRemoved)   /* Remove assignment if control register is removed */
        Timer_Delay_CONTROL |= Timer_Delay_CTRL_TRIG_EN;
    #endif
}

/*******************************************************************************
* Function Name: Timer_Delay_DisableTrigger
********************************************************************************
* 
* Summary:
*  Clears the control bit enabling Hardware Trigger mode
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_DisableTrigger(void) 
{
    #if (!Timer_Delay_ControlRegRemoved)   /* Remove assignment if control register is removed */
        Timer_Delay_CONTROL &= ~Timer_Delay_CTRL_TRIG_EN;
    #endif
}
#endif


#if(Timer_Delay_InterruptOnCaptureCount)
#if (!Timer_Delay_ControlRegRemoved)   /* Remove API if control register is removed */
/*******************************************************************************
* Function Name: Timer_Delay_SetInterruptCount
********************************************************************************
* 
* Summary:
*  This function sets the capture count before an interrupt is triggered.
*
* Parameters:  
*  interruptCount:  A value between 0 and 3 is valid.  If the value is 0, then 
*                   an interrupt will occur each time a capture occurs.  
*                   A value of 1 to 3 will cause the interrupt  
*                   to delay by the same number of captures.
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_SetInterruptCount(uint8 interruptCount) 
{
    /* This must only set to two bits of the control register associated */
    interruptCount &= Timer_Delay_CTRL_INTCNT_MASK;
    
    /* Clear the Current Setting */
    Timer_Delay_CONTROL &= ~Timer_Delay_CTRL_INTCNT_MASK;
    /* Write The New Setting */   
    Timer_Delay_CONTROL |= interruptCount;    
}
#endif /* Remove API if control register is removed */
#endif /* Timer_Delay_InterruptOnCaptureCount */


#if (Timer_Delay_UsingHWCaptureCounter)
/*******************************************************************************
* Function Name: Timer_Delay_SetCaptureCount
********************************************************************************
* 
* Summary:
*  This function sets the capture count
*
* Parameters:  
*  captureCount: A value between 2 and 127 inclusive is valid.  A value of 1 
*                to 127 will cause the interrupt to delay by the same number of 
*                captures.
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_SetCaptureCount(uint8 captureCount) 
{
    Timer_Delay_CAP_COUNT = captureCount;
}


/*******************************************************************************
* Function Name: Timer_Delay_ReadCaptureCount
********************************************************************************
* 
* Summary:
*  This function reads the capture count setting
*
* Parameters:  
*  void
*
* Return: 
*  Returns the Capture Count Setting
*
*******************************************************************************/
uint8 Timer_Delay_ReadCaptureCount(void) 
{
    return Timer_Delay_CAP_COUNT ;
}
#endif /* Timer_Delay_UsingHWCaptureCounter */


/*******************************************************************************
* Function Name: Timer_Delay_ClearFIFO
********************************************************************************
* 
* Summary:
*  This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void Timer_Delay_ClearFIFO(void) 
{
    while(Timer_Delay_ReadStatusRegister() & Timer_Delay_STATUS_FIFONEMP)
    {
        Timer_Delay_ReadCapture();
    }
}

#endif /* UDB Specific Functions */


/* [] END OF FILE */
