/*******************************************************************************
* File Name: Timer_LCD_PM.c  
* Version 2.0
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Timer_LCD.h"
static Timer_LCD_backupStruct Timer_LCD_backup;

/*******************************************************************************
* Function Name: Timer_LCD_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  Timer_LCD_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*    No
*
*******************************************************************************/
void Timer_LCD_SaveConfig(void)
{   
    #if (!Timer_LCD_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (Timer_LCD_PSOC3_ES2 || Timer_LCD_PSOC5_ES1)
            Timer_LCD_backup.TimerUdb = Timer_LCD_ReadCounter();
            Timer_LCD_backup.TimerPeriod = Timer_LCD_ReadPeriod();
            Timer_LCD_backup.InterruptMaskValue = Timer_LCD_STATUS_MASK;
            #if (Timer_LCD_UsingHWCaptureCounter)
                Timer_LCD_backup.TimerCaptureCounter = Timer_LCD_ReadCaptureCount();        
            #endif
        #endif
        
        #if (Timer_LCD_PSOC3_ES3 || Timer_LCD_PSOC5_ES2)
            Timer_LCD_backup.TimerUdb = Timer_LCD_ReadCounter();
            Timer_LCD_backup.InterruptMaskValue = Timer_LCD_STATUS_MASK;
            #if (Timer_LCD_UsingHWCaptureCounter)
                Timer_LCD_backup.TimerCaptureCounter = Timer_LCD_ReadCaptureCount();        
            #endif
        #endif
        
        #if(!Timer_LCD_ControlRegRemoved)
            Timer_LCD_backup.TimerControlRegister = Timer_LCD_ReadControlRegister();
        #endif
    #endif
}

/*******************************************************************************
* Function Name: Timer_LCD_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Timer_LCD_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void Timer_LCD_RestoreConfig(void) 
{
    #if (!Timer_LCD_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (Timer_LCD_PSOC3_ES2 || Timer_LCD_PSOC5_ES1)
            Timer_LCD_WriteCounter(Timer_LCD_backup.TimerUdb);
            Timer_LCD_WritePeriod(Timer_LCD_backup.TimerPeriod);
            Timer_LCD_STATUS_MASK =Timer_LCD_backup.InterruptMaskValue;
            #if (Timer_LCD_UsingHWCaptureCounter)
                Timer_LCD_SetCaptureCount(Timer_LCD_backup.TimerCaptureCounter);        
            #endif
        #endif
        
        #if (Timer_LCD_PSOC3_ES3 || Timer_LCD_PSOC5_ES2)
            Timer_LCD_WriteCounter(Timer_LCD_backup.TimerUdb);
            Timer_LCD_STATUS_MASK =Timer_LCD_backup.InterruptMaskValue;
            #if (Timer_LCD_UsingHWCaptureCounter)
                Timer_LCD_SetCaptureCount(Timer_LCD_backup.TimerCaptureCounter);           
            #endif
        #endif
        
        #if(!Timer_LCD_ControlRegRemoved)
            Timer_LCD_WriteControlRegister(Timer_LCD_backup.TimerControlRegister);
        #endif
    #endif
}


/*******************************************************************************
* Function Name: Timer_LCD_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Timer_LCD_backup.TimerEnableState:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*    No
*
*******************************************************************************/
void Timer_LCD_Sleep(void)
{
    #if(!Timer_LCD_ControlRegRemoved)
        /* Save Counter's enable state */
        if(Timer_LCD_CTRL_ENABLE == (Timer_LCD_CONTROL & Timer_LCD_CTRL_ENABLE))
        {
            /* Timer is enabled */
            Timer_LCD_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            Timer_LCD_backup.TimerEnableState = 0u;
        }
    #endif
    Timer_LCD_Stop();
    Timer_LCD_SaveConfig();
}


/*******************************************************************************
* Function Name: Timer_LCD_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Timer_LCD_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void Timer_LCD_Wakeup(void) 
{
    Timer_LCD_RestoreConfig();
    #if(!Timer_LCD_ControlRegRemoved)
        if(Timer_LCD_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                Timer_LCD_Enable();
        } /* Do nothing if Timer was disabled before */ 
    #endif
}


/* [] END OF FILE */
